@@ -19,7 +19,6 @@ Licensed under the terms of the MIT License
 # =============================================================================
 # Stdlib imports
 # =============================================================================
-from __future__ import print_function
 from collections import OrderedDict
 from enum import Enum
 import errno
@@ -48,7 +47,7 @@ requirements.check_spyder_kernels()
 from qtpy.compat import from_qvariant
 from qtpy.QtCore import (QCoreApplication, Qt, QTimer, Signal, Slot,
                          qInstallMessageHandler)
-from qtpy.QtGui import QColor, QKeySequence, QIcon
+from qtpy.QtGui import QColor, QKeySequence
 from qtpy.QtWidgets import (QApplication, QMainWindow, QMenu, QMessageBox,
                             QShortcut, QStyleFactory)
 
@@ -68,8 +67,10 @@ from qtawesome.iconic_font import FontError
 #==============================================================================
 from spyder import __version__
 from spyder import dependencies
+from spyder.app.find_plugins import (
+    find_external_plugins, find_internal_plugins)
 from spyder.app.utils import (
-    create_application, create_splash_screen, create_window,
+    create_application, create_splash_screen, create_window, ORIGINAL_SYS_EXIT,
     delete_debug_log_files, qt_message_handler, set_links_color, setup_logging,
     set_opengl_implementation)
 from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
@@ -91,7 +92,6 @@ from spyder.utils.palette import QStylePalette
 from spyder.utils.qthelpers import (create_action, add_actions, file_uri,
                                     qapplication, start_file)
 from spyder.utils.stylesheet import APP_STYLESHEET
-from spyder.app.find_plugins import find_external_plugins, find_internal_plugins
 
 # Spyder API Imports
 from spyder.api.exceptions import SpyderAPIError
@@ -115,9 +115,6 @@ if os.name == 'nt':
 # Module logger
 logger = logging.getLogger(__name__)
 
-# Keeping a reference to the original sys.exit before patching it
-ORIGINAL_SYS_EXIT = sys.exit
-
 # Get the cwd before initializing WorkingDirectory, which sets it to the one
 # used in the last session
 CWD = getcwd_or_home()
@@ -180,7 +177,7 @@ class MainWindow(QMainWindow):
         return PLUGIN_REGISTRY.is_plugin_enabled(plugin_name)
 
     def is_plugin_available(self, plugin_name):
-        """Determine if a given plugin is going to be loaded."""
+        """Determine if a given plugin is available."""
         return PLUGIN_REGISTRY.is_plugin_available(plugin_name)
 
     def show_status_message(self, message, timeout):
@@ -203,16 +200,6 @@ class MainWindow(QMainWindow):
         messageBox.setStandardButtons(QMessageBox.Ok)
         messageBox.show()
 
-    def add_plugin(self, plugin, external=False):
-        """
-        Add plugin to plugins dictionary.
-        """
-        self._PLUGINS[plugin.NAME] = plugin
-        if external:
-            self._EXTERNAL_PLUGINS[plugin.NAME] = plugin
-        else:
-            self._INTERNAL_PLUGINS[plugin.NAME] = plugin
-
     def register_plugin(self, plugin_name, external=False, omit_conf=False):
         """
         Register a plugin in Spyder Main Window.
@@ -263,8 +250,6 @@ class MainWindow(QMainWindow):
                 margin = CONF.get('main', 'custom_margin')
             plugin.update_margins(margin)
 
-        self.add_plugin(plugin, external=external)
-
         if plugin_name == Plugins.Shortcuts:
             for action, context, action_name in self.shortcut_queue:
                 self.register_shortcut(action, context, action_name)
@@ -353,7 +338,6 @@ class MainWindow(QMainWindow):
         logger.info("Removing {} dockwidget...".format(plugin.NAME))
         self.remove_dockwidget(plugin)
 
-        plugin.unregister()
         plugin._unregister()
 
     def create_plugin_conf_widget(self, plugin):
@@ -482,11 +466,15 @@ class MainWindow(QMainWindow):
                         plugin.NAME))
                 tabify_helper(plugin, next_to_plugins)
 
+                # Show external plugins
+                if plugin.NAME in PLUGIN_REGISTRY.external_plugins:
+                    plugin.get_widget().toggle_view(True)
+
             plugin.set_conf('enable', True)
             plugin.set_conf('first_time', False)
         else:
-            # This is needed to ensure new plugins are placed correctly
-            # without the need for a layout reset.
+            # This is needed to ensure plugins are placed correctly when
+            # switching layouts.
             logger.info("Tabify {} dockwidget...".format(plugin.NAME))
             # Check if plugin has no other dockwidgets in the same position
             if not bool(self.tabifiedDockWidgets(plugin.dockwidget)):
@@ -524,8 +512,9 @@ class MainWindow(QMainWindow):
         The `label` and `steps` keys allow customizing the content of the
         error dialog.
         """
-        if self.console:
-            self.console.handle_exception(error_data)
+        console = self.get_plugin(Plugins.Console, error=False)
+        if console:
+            console.handle_exception(error_data)
 
     def __init__(self, splash=None, options=None):
         QMainWindow.__init__(self)
@@ -599,9 +588,6 @@ class MainWindow(QMainWindow):
         # New API
         self._APPLICATION_TOOLBARS = OrderedDict()
         self._STATUS_WIDGETS = OrderedDict()
-        self._PLUGINS = OrderedDict()
-        self._EXTERNAL_PLUGINS = OrderedDict()
-        self._INTERNAL_PLUGINS = OrderedDict()
         # Mapping of new plugin identifiers vs old attributtes
         # names given for plugins or to prevent collisions with other
         # attributes, i.e layout (Qt) vs layout (SpyderPluginV2)
@@ -615,7 +601,7 @@ class MainWindow(QMainWindow):
             'projects': Plugins.Projects,
             'findinfiles': Plugins.Find,
             'layouts': Plugins.Layout,
-            }
+        }
 
         self.thirdparty_plugins = []
 
@@ -778,6 +764,8 @@ class MainWindow(QMainWindow):
             lambda plugin_name, omit_conf: self.register_plugin(
                 plugin_name, omit_conf=omit_conf))
 
+        PLUGIN_REGISTRY.set_main(self)
+
         # TODO: Remove circular dependency between help and ipython console
         # and remove this import. Help plugin should take care of it
         from spyder.plugins.help.utils.sphinxify import CSS_PATH, DARK_CSS_PATH
@@ -864,12 +852,20 @@ class MainWindow(QMainWindow):
 
         # Determine 'enable' config for the plugins that have it
         enabled_plugins = {}
+        registry_internal_plugins = {}
+        registry_external_plugins = {}
         for plugin in all_plugins.values():
             plugin_name = plugin.NAME
             plugin_main_attribute_name = (
                 self._INTERNAL_PLUGINS_MAPPING[plugin_name]
                 if plugin_name in self._INTERNAL_PLUGINS_MAPPING
                 else plugin_name)
+            if plugin_name in internal_plugins:
+                registry_internal_plugins[plugin_name] = (
+                    plugin_main_attribute_name, plugin)
+            else:
+                registry_external_plugins[plugin_name] = (
+                    plugin_main_attribute_name, plugin)
             try:
                 if CONF.get(plugin_main_attribute_name, "enable"):
                     enabled_plugins[plugin_name] = plugin
@@ -878,6 +874,9 @@ class MainWindow(QMainWindow):
                 enabled_plugins[plugin_name] = plugin
                 PLUGIN_REGISTRY.set_plugin_enabled(plugin_name)
 
+        PLUGIN_REGISTRY.set_all_internal_plugins(registry_internal_plugins)
+        PLUGIN_REGISTRY.set_all_external_plugins(registry_external_plugins)
+
         # Instantiate internal Spyder 5 plugins
         for plugin_name in internal_plugins:
             if plugin_name in enabled_plugins:
@@ -891,14 +890,8 @@ class MainWindow(QMainWindow):
             if plugin_name in enabled_plugins:
                 PluginClass = internal_plugins[plugin_name]
                 if issubclass(PluginClass, SpyderPlugin):
-                    if plugin_name == Plugins.IPythonConsole:
-                        plugin_instance = PLUGIN_REGISTRY.register_plugin(
-                            self, PluginClass, external=False)
-                        plugin_instance.sig_exception_occurred.connect(
-                            self.handle_exception)
-                    else:
-                        plugin_instance = PLUGIN_REGISTRY.register_plugin(
-                            self, PluginClass, external=False)
+                    plugin_instance = PLUGIN_REGISTRY.register_plugin(
+                        self, PluginClass, external=False)
                     self.preferences.register_plugin_preferences(
                         plugin_instance)
 
@@ -909,15 +902,6 @@ class MainWindow(QMainWindow):
                 try:
                     plugin_instance = PLUGIN_REGISTRY.register_plugin(
                         self, PluginClass, external=True)
-
-                    # These attributes come from spyder.app.find_plugins to
-                    # add plugins to the dependencies dialog
-                    module = PluginClass._spyder_module_name
-                    package_name = PluginClass._spyder_package_name
-                    version = PluginClass._spyder_version
-                    description = plugin_instance.get_description()
-                    dependencies.add(module, package_name, description,
-                                     version, None, kind=dependencies.PLUGIN)
                 except Exception as error:
                     print("%s: %s" % (PluginClass, str(error)), file=STDERR)
                     traceback.print_exc(file=STDERR)
@@ -1016,7 +1000,10 @@ class MainWindow(QMainWindow):
         self.edit_menu_actions += [self.undo_action, self.redo_action,
                                    None, self.cut_action, self.copy_action,
                                    self.paste_action, self.selectall_action,
-                                   None] + self.editor.edit_menu_actions
+                                   None]
+        if self.get_plugin(Plugins.Editor, error=False):
+            self.edit_menu_actions += self.editor.edit_menu_actions
+
         switcher_actions = [
             self.file_switcher_action,
             self.symbol_finder_action
@@ -1095,9 +1082,10 @@ class MainWindow(QMainWindow):
         """
         # Mapping of new plugin identifiers vs old attributtes
         # names given for plugins
-        if attr in self._INTERNAL_PLUGINS_MAPPING.keys():
-            return self.get_plugin(self._INTERNAL_PLUGINS_MAPPING[attr])
         try:
+            if attr in self._INTERNAL_PLUGINS_MAPPING.keys():
+                return self.get_plugin(
+                    self._INTERNAL_PLUGINS_MAPPING[attr], error=False)
             return self.get_plugin(attr)
         except SpyderAPIError:
             pass
@@ -1110,12 +1098,6 @@ class MainWindow(QMainWindow):
         The actions here are related with setting up the main window.
         """
         logger.info("Setting up window...")
-        # Create external plugins before loading the layout to include them in
-        # the window restore state after restarts.
-        for plugin, plugin_instance in self._EXTERNAL_PLUGINS.items():
-            self.tabify_plugin(plugin_instance, Plugins.Console)
-            if isinstance(plugin_instance, SpyderDockablePlugin):
-                plugin_instance.get_widget().toggle_view(False)
 
         for plugin_name in PLUGIN_REGISTRY:
             plugin_instance = PLUGIN_REGISTRY.get_plugin(plugin_name)
@@ -1124,6 +1106,16 @@ class MainWindow(QMainWindow):
             except AttributeError:
                 pass
 
+        # Tabify external plugins which were installed after Spyder was
+        # installed.
+        # Note: This is only necessary the first time a plugin is loaded.
+        # Afterwwrds, the plugin placement is recorded on the window hexstate,
+        # which is loaded by the layouts plugin during the next session.
+        for plugin_name in PLUGIN_REGISTRY.external_plugins:
+            plugin_instance = PLUGIN_REGISTRY.get_plugin(plugin_name)
+            if plugin_instance.get_conf('first_time', True):
+                self.tabify_plugin(plugin_instance, Plugins.Console)
+
         if self.splash is not None:
             self.splash.hide()
 
@@ -1137,7 +1129,8 @@ class MainWindow(QMainWindow):
                     pass
 
         # Register custom layouts
-        for plugin, plugin_instance in self._PLUGINS.items():
+        for plugin_name in PLUGIN_REGISTRY.external_plugins:
+            plugin_instance = PLUGIN_REGISTRY.get_plugin(plugin_name)
             if hasattr(plugin_instance, 'CUSTOM_LAYOUTS'):
                 if isinstance(plugin_instance.CUSTOM_LAYOUTS, list):
                     for custom_layout in plugin_instance.CUSTOM_LAYOUTS:
@@ -1147,7 +1140,7 @@ class MainWindow(QMainWindow):
                     logger.info(
                         'Unable to load custom layouts for {}. '
                         'Expecting a list of layout classes but got {}'
-                        .format(plugin, plugin_instance.CUSTOM_LAYOUTS)
+                        .format(plugin_name, plugin_instance.CUSTOM_LAYOUTS)
                     )
         self.layouts.update_layout_menu_actions()
 
@@ -1160,6 +1153,13 @@ class MainWindow(QMainWindow):
         Actions to be performed only after the main window's `show` method
         is triggered.
         """
+        # Required plugins
+        help_plugin = self.get_plugin(Plugins.Help, error=False)
+        ipyconsole = self.get_plugin(Plugins.IPythonConsole, error=False)
+        projects = self.get_plugin(Plugins.Projects, error=False)
+        editor = self.get_plugin(Plugins.Editor, error=False)
+        console = self.get_plugin(Plugins.Console, error=False)
+
         # Process pending events and hide splash before loading the
         # previous session.
         QApplication.processEvents()
@@ -1199,14 +1199,16 @@ class MainWindow(QMainWindow):
 
         # Hide Internal Console so that people don't use it instead of
         # the External or IPython ones
-        if self.console.dockwidget.isVisible() and DEV is None:
-            self.console.toggle_view_action.setChecked(False)
-            self.console.dockwidget.hide()
+        if console and console.dockwidget.isVisible() and DEV is None:
+            console.toggle_view_action.setChecked(False)
+            console.dockwidget.hide()
 
-        # Show Help and Consoles by default
-        plugins_to_show = [self.ipyconsole]
-        if self.help is not None:
-            plugins_to_show.append(self.help)
+        # Show Help and IPython console by default
+        plugins_to_show = []
+        if help_plugin:
+            plugins_to_show.append(help_plugin)
+        if ipyconsole:
+            plugins_to_show.append(ipyconsole)
         for plugin in plugins_to_show:
             if plugin.dockwidget.isVisible():
                 plugin.dockwidget.raise_()
@@ -1219,17 +1221,20 @@ class MainWindow(QMainWindow):
         # line options to the plugins.
         if self.open_project:
             if not running_in_mac_app():
-                self.projects.open_project(
-                    self.open_project, workdir=self.init_workdir
-                )
+                if projects:
+                    projects.open_project(
+                        self.open_project, workdir=self.init_workdir
+                    )
         else:
             # Load last project if a project was active when Spyder
             # was closed
-            self.projects.reopen_last_project()
+            if projects:
+                projects.reopen_last_project()
 
             # If no project is active, load last session
-            if self.projects.get_active_project() is None:
-                self.editor.setup_open_files(close_previous_files=False)
+            if projects and projects.get_active_project() is None:
+                if editor:
+                    editor.setup_open_files(close_previous_files=False)
 
         # Raise the menuBar to the top of the main window widget's stack
         # Fixes spyder-ide/spyder#3887.
@@ -1241,10 +1246,27 @@ class MainWindow(QMainWindow):
             assert 'pandas' not in sys.modules
             assert 'matplotlib' not in sys.modules
 
+        # Restore undocked plugins
+        self.restore_undocked_plugins()
+
         # Notify that the setup of the mainwindow was finished
         self.is_setting_up = False
         self.sig_setup_finished.emit()
 
+    def restore_undocked_plugins(self):
+        """Restore plugins that were undocked in the previous session."""
+        logger.info("Restoring undocked plugins from the previous session")
+
+        for plugin_name in PLUGIN_REGISTRY:
+            plugin = PLUGIN_REGISTRY.get_plugin(plugin_name)
+            if isinstance(plugin, SpyderDockablePlugin):
+                if plugin.get_conf('undocked_on_window_close', default=False):
+                    plugin.get_widget().create_window()
+            elif isinstance(plugin, SpyderPluginWidget):
+                if plugin.get_option('undocked_on_window_close',
+                                     default=False):
+                    plugin._create_window()
+
     def set_window_title(self):
         """Set window title."""
         if DEV is not None:
@@ -1265,8 +1287,9 @@ class MainWindow(QMainWindow):
 
         # TODO: Remove self.projects reference once there's an API for setting
         # window title.
-        if self.projects is not None:
-            path = self.projects.get_active_project_path()
+        projects = self.get_plugin(Plugins.Projects, error=False)
+        if projects:
+            path = projects.get_active_project_path()
             if path:
                 path = path.replace(get_home_dir(), u'~')
                 title = u'{0} - {1}'.format(path, title)
@@ -1278,13 +1301,15 @@ class MainWindow(QMainWindow):
     # plugins
     def register_shortcut(self, qaction_or_qshortcut, context, name,
                           add_shortcut_to_tip=True, plugin_name=None):
-        self.shortcuts.register_shortcut(
-            qaction_or_qshortcut,
-            context,
-            name,
-            add_shortcut_to_tip=add_shortcut_to_tip,
-            plugin_name=plugin_name,
-        )
+        shortcuts = self.get_plugin(Plugins.Shortcuts, error=False)
+        if shortcuts:
+            shortcuts.register_shortcut(
+                qaction_or_qshortcut,
+                context,
+                name,
+                add_shortcut_to_tip=add_shortcut_to_tip,
+                plugin_name=plugin_name,
+            )
 
     # --- Other
     def update_source_menu(self):
@@ -1292,7 +1317,9 @@ class MainWindow(QMainWindow):
         # This is necessary to avoid an error at startup.
         # Fixes spyder-ide/spyder#14901
         try:
-            self.editor.refresh_formatter_name()
+            editor = self.get_plugin(Plugins.Editor, error=False)
+            if editor:
+                editor.refresh_formatter_name()
         except AttributeError:
             pass
 
@@ -1374,10 +1401,11 @@ class MainWindow(QMainWindow):
         #     instance
         console, not_readonly, readwrite_editor = textedit_properties
 
-        # Editor has focus and there is no file opened in it
-        if (not console and not_readonly and self.editor
-                and not self.editor.is_file_opened()):
-            return
+        if hasattr(self, 'editor'):
+            # Editor has focus and there is no file opened in it
+            if (not console and not_readonly and self.editor
+                    and not self.editor.is_file_opened()):
+                return
 
         # Disabling all actions to begin with
         for child in self.edit_menu.actions():
@@ -1400,7 +1428,7 @@ class MainWindow(QMainWindow):
         # Comment, uncomment, indent, unindent...
         if not console and not_readonly:
             # This is the editor and current file is writable
-            if self.editor:
+            if self.get_plugin(Plugins.Editor, error=False):
                 for action in self.editor.edit_menu_actions:
                     action.setEnabled(True)
 
@@ -1465,10 +1493,10 @@ class MainWindow(QMainWindow):
 
     def moveEvent(self, event):
         """Reimplement Qt method"""
-        if not self.isMaximized() and not self.layouts.get_fullscreen_flag():
-            self.window_position = self.pos()
+        if hasattr(self, 'layouts'):
+            if not self.isMaximized() and not self.layouts.get_fullscreen_flag():
+                self.window_position = self.pos()
         QMainWindow.moveEvent(self, event)
-
         # To be used by the tour to be able to move
         self.sig_moved.emit(event)
 
@@ -1501,7 +1529,7 @@ class MainWindow(QMainWindow):
 
         self.previous_focused_widget =  old
 
-    def closing(self, cancelable=False):
+    def closing(self, cancelable=False, close_immediately=False):
         """Exit tasks"""
         if self.already_closed or self.is_starting_up:
             return True
@@ -1516,41 +1544,18 @@ class MainWindow(QMainWindow):
         if CONF.get('main', 'single_instance') and self.open_files_server:
             self.open_files_server.close()
 
-        # Internal plugins
-        for plugin in (self.widgetlist + self.thirdparty_plugins):
-            # New API
-            try:
-                if isinstance(plugin, SpyderDockablePlugin):
-                    plugin.close_window()
-                if not plugin.on_close(cancelable):
-                    return False
-            except AttributeError:
-                pass
+        can_close = PLUGIN_REGISTRY.delete_all_plugins(
+            excluding={Plugins.Layout}, close_immediately=close_immediately)
 
-            # Old API
-            try:
-                plugin._close_window()
-                if not plugin.closing_plugin(cancelable):
-                    return False
-            except AttributeError:
-                pass
-
-        # New API: External plugins
-        for plugin_name, plugin in self._EXTERNAL_PLUGINS.items():
-            try:
-                if isinstance(plugin, SpyderDockablePlugin):
-                    plugin.close_window()
-
-                if not plugin.on_close(cancelable):
-                    return False
-            except AttributeError as e:
-                logger.error(str(e))
+        if not can_close and not close_immediately:
+            return False
 
         # Save window settings *after* closing all plugin windows, in order
         # to show them in their previous locations in the next session.
         # Fixes spyder-ide/spyder#12139
         prefix = 'window' + '/'
         self.layouts.save_current_window_settings(prefix)
+        PLUGIN_REGISTRY.delete_plugin(Plugins.Layout)
 
         self.already_closed = True
         return True
@@ -1572,7 +1577,6 @@ class MainWindow(QMainWindow):
                 self.addDockWidget(location, dockwidget)
                 self.widgetlist.append(plugin)
 
-    @Slot()
     def global_callback(self):
         """Global callback"""
         widget = QApplication.focusWidget()
@@ -1587,10 +1591,12 @@ class MainWindow(QMainWindow):
             return
 
     def redirect_internalshell_stdio(self, state):
-        if state:
-            self.console.redirect_stds()
-        else:
-            self.console.restore_stds()
+        console = self.get_plugin(Plugins.Console, error=False)
+        if console:
+            if state:
+                console.redirect_stds()
+            else:
+                console.restore_stds()
 
     def open_external_console(self, fname, wdir, args, interact, debug, python,
                               python_args, systerm, post_mortem=False):
@@ -1619,10 +1625,15 @@ class MainWindow(QMainWindow):
         """
         fname = to_text_string(fname)
         ext = osp.splitext(fname)[1]
+        editor = self.get_plugin(Plugins.Editor, error=False)
+        variableexplorer = self.get_plugin(
+            Plugins.VariableExplorer, error=False)
+
         if encoding.is_text_file(fname):
-            self.editor.load(fname)
-        elif self.variableexplorer is not None and ext in IMPORT_EXT:
-            self.variableexplorer.import_data(fname)
+            if editor:
+                editor.load(fname)
+        elif variableexplorer is not None and ext in IMPORT_EXT:
+            variableexplorer.get_widget().import_data(fname)
         elif not external:
             fname = file_uri(fname)
             start_file(fname)
@@ -1748,7 +1759,12 @@ class MainWindow(QMainWindow):
     def show_path_manager(self):
         """Show path manager dialog."""
         from spyder.widgets.pathmanager import PathManager
-        read_only_path = tuple(self.projects.get_pythonpath())
+        projects = self.get_plugin(Plugins.Projects, error=False)
+
+        read_only_path = ()
+        if projects:
+            read_only_path = tuple(projects.get_pythonpath())
+
         dialog = PathManager(self, self.path, read_only_path,
                              self.not_active_path, sync=True)
         self._path_manager = dialog
@@ -1758,7 +1774,11 @@ class MainWindow(QMainWindow):
 
     def pythonpath_changed(self):
         """Project's PYTHONPATH contribution has changed."""
-        self.project_path = tuple(self.projects.get_pythonpath())
+        projects = self.get_plugin(Plugins.Projects, error=False)
+
+        self.project_path = ()
+        if projects:
+            self.project_path = tuple(projects.get_pythonpath())
         path_dict = self.get_spyder_pythonpath_dict()
         self.update_python_path(path_dict)
 
@@ -1843,6 +1863,25 @@ class MainWindow(QMainWindow):
             self.sig_open_external_file.emit(fname)
             req.sendall(b' ')
 
+    # ---- Quit and restart, and reset spyder defaults
+    @Slot()
+    def reset_spyder(self):
+        """
+        Quit and reset Spyder and then Restart application.
+        """
+        answer = QMessageBox.warning(self, _("Warning"),
+             _("Spyder will restart and reset to default settings: <br><br>"
+               "Do you want to continue?"),
+             QMessageBox.Yes | QMessageBox.No)
+        if answer == QMessageBox.Yes:
+            self.restart(reset=True)
+
+    @Slot()
+    def restart(self, reset=False, close_immediately=False):
+        """Wrapper to handle plugins request to restart Spyder."""
+        self.application.restart(
+            reset=reset, close_immediately=close_immediately)
+
     # ---- Global Switcher
     def open_switcher(self, symbol=False):
         """Open switcher dialog box."""
