@@ -188,9 +188,6 @@ class AppearanceConfigPage(PluginConfigPage):
 
             if 'ui_theme' not in self.changed_options:
                 self.main.editor.apply_plugin_settings(['color_scheme_name'])
-                if self.main.ipyconsole is not None:
-                    self.main.ipyconsole.apply_plugin_settings(
-                        ['color_scheme_name'])
 
                 for plugin in self.main.thirdparty_plugins:
                     try:
@@ -210,9 +207,6 @@ class AppearanceConfigPage(PluginConfigPage):
 
             if 'ui_theme' not in self.changed_options:
                 self.main.editor.apply_plugin_settings(['color_scheme_name'])
-                if self.main.ipyconsole is not None:
-                    self.main.ipyconsole.apply_plugin_settings(
-                        ['color_scheme_name'])
 
                 for plugin in self.main.thirdparty_plugins:
                     try:
