@@ -21,6 +21,7 @@ import shutil
 import sys
 
 # Third party imports
+from qtpy import PYQT5
 from qtpy.compat import getexistingdirectory, getsavefilename
 from qtpy.QtCore import QDir, QMimeData, Qt, QTimer, QUrl, Signal, Slot
 from qtpy.QtGui import QDrag
@@ -249,7 +250,11 @@ class DirView(QTreeView, SpyderWidgetMixin):
         parent: QWidget
             Parent QWidget of the widget.
         """
-        super().__init__(parent=parent, class_parent=parent)
+        if PYQT5:
+            super().__init__(parent=parent, class_parent=parent)
+        else:
+            QTreeView.__init__(self, parent)
+            SpyderWidgetMixin.__init__(self, class_parent=parent)
 
         # Attributes
         self._parent = parent
@@ -562,7 +567,8 @@ class DirView(QTreeView, SpyderWidgetMixin):
         self.context_menu.aboutToShow.connect(self.update_actions)
 
     @on_conf_change(option=['size_column', 'type_column', 'date_column',
-                            'name_filters', 'show_hidden'])
+                            'name_filters', 'show_hidden',
+                            'single_click_to_open'])
     def on_conf_update(self, option, value):
         if option == 'size_column':
             self.setColumnHidden(DirViewColumns.Size, not value)
@@ -574,7 +580,9 @@ class DirView(QTreeView, SpyderWidgetMixin):
             if self.filter_on:
                 self.filter_files(value)
         elif option == 'show_hidden':
-            self.set_show_hidden(self.get_conf('show_hidden'))
+            self.set_show_hidden(value)
+        elif option == 'single_click_to_open':
+            self.set_single_click_to_open(value)
 
     def update_actions(self):
         fnames = self.get_selected_filenames()
@@ -604,8 +612,9 @@ class DirView(QTreeView, SpyderWidgetMixin):
         self.open_interpreter_action.setVisible(only_dirs)
         self.open_with_spyder_action.setVisible(only_files and only_valid)
         self.open_with_submenu.menuAction().setVisible(False)
-        self.paste_action.setDisabled(
-            not QApplication.clipboard().mimeData().hasUrls())
+        clipboard = QApplication.clipboard()
+        has_urls = clipboard.mimeData().hasUrls()
+        self.paste_action.setDisabled(not has_urls)
 
         # VCS support is quite limited for now, so we are enabling the VCS
         # related actions only when a single file/folder is selected:
@@ -775,6 +784,21 @@ class DirView(QTreeView, SpyderWidgetMixin):
         if self.get_conf('single_click_to_open'):
             self.clicked(index=self.indexAt(event.pos()))
 
+    def mouseMoveEvent(self, event):
+        """Change cursor shape when using single_click_to_open option."""
+        index = self.indexAt(event.pos())
+        if index.isValid():
+            vrect = self.visualRect(index)
+            item_identation = vrect.x() - self.visualRect(self.rootIndex()).x()
+            if event.pos().x() > item_identation:
+                # When hovering over directories or files
+                self.setCursor(Qt.PointingHandCursor)
+            else:
+                # On every other element
+                self.setCursor(Qt.ArrowCursor)
+
+        super().mouseMoveEvent(event)
+
     def dragEnterEvent(self, event):
         """Drag and Drop - Enter event"""
         event.setAccepted(event.mimeData().hasFormat("text/plain"))
@@ -1577,7 +1601,13 @@ class DirView(QTreeView, SpyderWidgetMixin):
     # ------------------------------------------------------------------------
     def set_single_click_to_open(self, value):
         """Set single click to open items."""
-        self.set_conf('single_click_to_open', value)
+        # Track mouse movements to change cursor shape when the option is
+        # True. This activates the mouseMoveEvent declared above.
+        if value:
+            self.setMouseTracking(True)
+        else:
+            self.setMouseTracking(False)
+            self.unsetCursor()
 
     def set_file_associations(self, value):
         """Set file associations open items."""
