@@ -12,19 +12,18 @@ Help Plugin.
 import os
 
 # Third party imports
-from qtpy.QtCore import Qt, Signal
+from qtpy.QtCore import Signal
 
 # Local imports
-from spyder import __docs_url__, __forum_url__, __trouble_url__
+from spyder.api.exceptions import SpyderAPIError
 from spyder.api.plugins import Plugins, SpyderDockablePlugin
-from spyder.api.plugin_registration.decorators import on_plugin_available
+from spyder.api.plugin_registration.decorators import (
+    on_plugin_available, on_plugin_teardown)
 from spyder.api.translations import get_translation
 from spyder.config.base import get_conf_path
 from spyder.config.fonts import DEFAULT_SMALL_DELTA
-from spyder.plugins.console.api import ConsoleActions
 from spyder.plugins.help.confpage import HelpConfigPage
 from spyder.plugins.help.widgets import HelpWidget
-from spyder.utils.qthelpers import start_file
 
 # Localization
 _ = get_translation('spyder')
@@ -62,7 +61,8 @@ class Help(SpyderDockablePlugin):
 
     # --- SpyderDocakblePlugin API
     #  -----------------------------------------------------------------------
-    def get_name(self):
+    @staticmethod
+    def get_name():
         return _('Help')
 
     def get_description(self):
@@ -107,8 +107,7 @@ class Help(SpyderDockablePlugin):
         ipyconsole = self.get_plugin(Plugins.IPythonConsole)
 
         ipyconsole.sig_shellwidget_changed.connect(self.set_shellwidget)
-        ipyconsole.sig_shellwidget_created.connect(
-            self.set_shellwidget)
+        ipyconsole.sig_shellwidget_created.connect(self.set_shellwidget)
         ipyconsole.sig_render_plain_text_requested.connect(
             self.show_plain_text)
         ipyconsole.sig_render_rich_text_requested.connect(
@@ -126,8 +125,8 @@ class Help(SpyderDockablePlugin):
         shortcuts = self.get_plugin(Plugins.Shortcuts)
 
         # See: spyder-ide/spyder#6992
-        shortcuts.sig_shortcuts_updated.connect(
-            lambda: self.show_intro_message())
+        self._show_intro_message = lambda: self.show_intro_message()
+        shortcuts.sig_shortcuts_updated.connect(self._show_intro_message)
 
         if self.is_plugin_available(Plugins.MainMenu):
             self._setup_menus()
@@ -140,6 +139,46 @@ class Help(SpyderDockablePlugin):
         else:
             self._setup_menus()
 
+    @on_plugin_teardown(plugin=Plugins.Console)
+    def on_console_teardown(self):
+        widget = self.get_widget()
+        internal_console = self.get_plugin(Plugins.Console)
+        internal_console.sig_help_requested.disconnect(self.set_object_text)
+        widget.set_internal_console(None)
+
+    @on_plugin_teardown(plugin=Plugins.Editor)
+    def on_editor_teardown(self):
+        editor = self.get_plugin(Plugins.Editor)
+        editor.sig_help_requested.disconnect(self.set_editor_doc)
+
+    @on_plugin_teardown(plugin=Plugins.IPythonConsole)
+    def on_ipython_console_teardown(self):
+        ipyconsole = self.get_plugin(Plugins.IPythonConsole)
+
+        ipyconsole.sig_shellwidget_changed.disconnect(self.set_shellwidget)
+        ipyconsole.sig_shellwidget_created.disconnect(
+            self.set_shellwidget)
+        ipyconsole.sig_render_plain_text_requested.disconnect(
+            self.show_plain_text)
+        ipyconsole.sig_render_rich_text_requested.disconnect(
+            self.show_rich_text)
+
+        ipyconsole.sig_help_requested.disconnect(self.set_object_text)
+
+    @on_plugin_teardown(plugin=Plugins.Preferences)
+    def on_preferences_teardown(self):
+        preferences = self.get_plugin(Plugins.Preferences)
+        preferences.deregister_plugin_preferences(self)
+
+    @on_plugin_teardown(plugin=Plugins.Shortcuts)
+    def on_shortcuts_teardown(self):
+        shortcuts = self.get_plugin(Plugins.Shortcuts)
+        shortcuts.sig_shortcuts_updated.disconnect(self._show_intro_message)
+
+    @on_plugin_teardown(plugin=Plugins.MainMenu)
+    def on_main_menu_teardown(self):
+        self._remove_menus()
+
     def update_font(self):
         color_scheme = self.get_color_scheme()
         font = self.get_font()
@@ -162,12 +201,11 @@ class Help(SpyderDockablePlugin):
             widget.set_plain_text_color_scheme(self.get_color_scheme())
 
         # To make auto-connection changes take place instantly
-        editor = self.get_plugin(Plugins.Editor)
-        editor.apply_plugin_settings({'connect_to_oi'})
-
-        ipyconsole = self.get_plugin(Plugins.IPythonConsole)
-        if ipyconsole:
-            ipyconsole.apply_plugin_settings({'connect_to_oi'})
+        try:
+            editor = self.get_plugin(Plugins.Editor)
+            editor.apply_plugin_settings({'connect_to_oi'})
+        except SpyderAPIError:
+            pass
 
     # --- Private API
     # ------------------------------------------------------------------------
@@ -189,6 +227,13 @@ class Help(SpyderDockablePlugin):
                 before=shortcuts_summary_action,
                 before_section=HelpMenuSections.Support)
 
+    def _remove_menus(self):
+        from spyder.plugins.mainmenu.api import ApplicationMenus
+        mainmenu = self.get_plugin(Plugins.MainMenu)
+        mainmenu.remove_item_from_application_menu(
+            HelpActions.ShowSpyderTutorialAction,
+            menu_id=ApplicationMenus.Help)
+
     # --- Public API
     # ------------------------------------------------------------------------
     def set_shellwidget(self, shellwidget):
