@@ -37,7 +37,7 @@ import sympy
 # Local imports
 from spyder.config.base import get_home_dir, running_in_ci
 from spyder.config.gui import get_color_scheme
-from spyder.config.manager import CONF
+from spyder.config.manager import ConfigurationManager
 from spyder.py3compat import PY2, to_text_string
 from spyder.plugins.help.tests.test_plugin import check_text
 from spyder.plugins.help.utils.sphinxify import CSS_PATH
@@ -92,21 +92,24 @@ def get_conda_test_env(test_env_name=u'spytest-ž'):
 # Qt Test Fixtures
 # =============================================================================
 @pytest.fixture
-def ipyconsole(qtbot, request):
+def ipyconsole(qtbot, request, tmpdir):
     """IPython console fixture."""
+    configuration = ConfigurationManager(conf_path=str(tmpdir))
 
     class MainWindowMock(QMainWindow):
         def get_spyder_pythonpath(self):
-            return CONF.get('main', 'spyder_pythonpath', [])
+            return configuration.get('main', 'spyder_pythonpath', [])
 
         def __getattr__(self, attr):
             if attr == 'consoles_menu_actions':
                 return []
+            elif attr == 'editor':
+                return None
             else:
                 return Mock()
 
     # Tests assume inline backend
-    CONF.set('ipython_console', 'pylab/backend', 0)
+    configuration.set('ipython_console', 'pylab/backend', 0)
 
     # Start in a new working directory the console
     use_startup_wdir = request.node.get_closest_marker('use_startup_wdir')
@@ -114,30 +117,36 @@ def ipyconsole(qtbot, request):
         new_wdir = osp.join(os.getcwd(), NEW_DIR)
         if not osp.exists(new_wdir):
             os.mkdir(new_wdir)
-        CONF.set('workingdir', 'console/use_fixed_directory', True)
-        CONF.set('workingdir', 'console/fixed_directory', new_wdir)
+        configuration.set('workingdir', 'console/use_fixed_directory', True)
+        configuration.set('workingdir', 'console/fixed_directory', new_wdir)
     else:
-        CONF.set('workingdir', 'console/use_fixed_directory', False)
-        CONF.set('workingdir', 'console/fixed_directory', get_home_dir())
+        configuration.set('workingdir', 'console/use_fixed_directory', False)
+        configuration.set(
+            'workingdir', 'console/fixed_directory', get_home_dir())
 
     # Test the console with a non-ascii temp dir
     non_ascii_dir = request.node.get_closest_marker('non_ascii_dir')
     if non_ascii_dir:
         test_dir = NON_ASCII_DIR
     else:
-        test_dir = None
+        test_dir = ''
 
     # Instruct the console to not use a stderr file
     no_stderr_file = request.node.get_closest_marker('no_stderr_file')
     if no_stderr_file:
-        test_no_stderr = True
+        test_no_stderr = 'True'
     else:
-        test_no_stderr = False
+        test_no_stderr = ''
 
     # Use the automatic backend if requested
     auto_backend = request.node.get_closest_marker('auto_backend')
     if auto_backend:
-        CONF.set('ipython_console', 'pylab/backend', 1)
+        configuration.set('ipython_console', 'pylab/backend', 1)
+
+    # Use the Tkinter backend if requested
+    tk_backend = request.node.get_closest_marker('tk_backend')
+    if tk_backend:
+        configuration.set('ipython_console', 'pylab/backend', 8)
 
     # Start a Pylab client if requested
     pylab_client = request.node.get_closest_marker('pylab_client')
@@ -154,41 +163,41 @@ def ipyconsole(qtbot, request):
     # Use an external interpreter if requested
     external_interpreter = request.node.get_closest_marker('external_interpreter')
     if external_interpreter:
-        CONF.set('main_interpreter', 'default', False)
-        CONF.set('main_interpreter', 'executable', sys.executable)
+        configuration.set('main_interpreter', 'default', False)
+        configuration.set('main_interpreter', 'executable', sys.executable)
     else:
-        CONF.set('main_interpreter', 'default', True)
-        CONF.set('main_interpreter', 'executable', '')
+        configuration.set('main_interpreter', 'default', True)
+        configuration.set('main_interpreter', 'executable', '')
 
     # Use the test environment interpreter if requested
     test_environment_interpreter = request.node.get_closest_marker(
         'test_environment_interpreter')
 
     if test_environment_interpreter:
-        CONF.set('main_interpreter', 'default', False)
-        CONF.set('main_interpreter', 'executable', get_conda_test_env())
+        configuration.set('main_interpreter', 'default', False)
+        configuration.set(
+            'main_interpreter', 'executable', get_conda_test_env())
     else:
-        CONF.set('main_interpreter', 'default', True)
-        CONF.set('main_interpreter', 'executable', '')
+        configuration.set('main_interpreter', 'default', True)
+        configuration.set('main_interpreter', 'executable', '')
 
     # Conf css_path in the Appeareance plugin
-    CONF.set('appearance', 'css_path', CSS_PATH)
+    configuration.set('appearance', 'css_path', CSS_PATH)
 
-    # Create the console and a new client
+    # Create the console and a new client and set environment
+    os.environ['IPYCONSOLE_TESTING'] = 'True'
+    os.environ['IPYCONSOLE_TEST_DIR'] = test_dir
+    os.environ['IPYCONSOLE_TEST_NO_STDERR'] = test_no_stderr
     window = MainWindowMock()
-    console = IPythonConsole(parent=window,
-                             testing=True,
-                             test_dir=test_dir,
-                             test_no_stderr=test_no_stderr)
-    console.dockwidget = Mock()
-    console._toggle_view_action = Mock()
+    console = IPythonConsole(parent=window, configuration=configuration)
+    console._register()
     console.create_new_client(is_pylab=is_pylab,
                               is_sympy=is_sympy,
                               is_cython=is_cython)
-    window.setCentralWidget(console)
+    window.setCentralWidget(console.get_widget())
 
     # Set exclamation mark to True
-    CONF.set('ipython_console', 'pdb_use_exclamation_mark', True)
+    configuration.set('ipython_console', 'pdb_use_exclamation_mark', True)
 
     # This segfaults on macOS
     if not sys.platform == "darwin":
@@ -210,9 +219,11 @@ def ipyconsole(qtbot, request):
                 print(client.info_page)
 
     # Close
-    console.closing_plugin()
-    console.close()
+    console.on_close()
     window.close()
+    os.environ.pop('IPYCONSOLE_TESTING')
+    os.environ.pop('IPYCONSOLE_TEST_DIR')
+    os.environ.pop('IPYCONSOLE_TEST_NO_STDERR')
 
 
 # =============================================================================
@@ -229,6 +240,10 @@ def test_banners(ipyconsole, qtbot):
 
     # Long banner
     text = control.toPlainText().splitlines()
+    if "Update LANGUAGE_CODES" in text[0]:
+        text = text[1:]
+        while not text[0].strip():
+            text = text[1:]
     py_ver = sys.version.splitlines()[0].strip()
     assert py_ver in text[0]  # Python version in first line
     assert 'license' in text[1]  # 'license' mention in second line
@@ -294,21 +309,39 @@ def test_get_calltips(ipyconsole, qtbot, function, signature, documentation):
 @flaky(max_runs=3)
 @pytest.mark.auto_backend
 def test_auto_backend(ipyconsole, qtbot):
-    """Test that the automatic backend is working correctly."""
+    """Test that the automatic backend was set correctly."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
 
     with qtbot.waitSignal(shell.executed):
-        shell.execute("import matplotlib; matplotlib.get_backend()")
+        shell.execute("ip = get_ipython(); ip.kernel.eventloop")
 
     # Assert there are no errors in the console and we set the right
     # backend.
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'NOTE' not in control.toPlainText()
     assert 'Error' not in control.toPlainText()
-    assert 'Qt5Agg' in control.toPlainText()
+    assert 'loop_qt5' in control.toPlainText()
+
+
+@flaky(max_runs=3)
+@pytest.mark.tk_backend
+@pytest.mark.skipif(os.name == 'nt', reason="Fails on Windows")
+def test_tk_backend(ipyconsole, qtbot):
+    """Test that the Tkinter backend was set correctly."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("ip = get_ipython(); ip.kernel.eventloop")
+
+    # Assert we set the right backend in the kernel.
+    control = ipyconsole.get_widget().get_focus_widget()
+    assert 'loop_tk' in control.toPlainText()
 
 
 @flaky(max_runs=3)
@@ -325,7 +358,7 @@ def test_pylab_client(ipyconsole, qtbot):
         shell.execute("e")
 
     # Assert there are no errors in the console
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'Error' not in control.toPlainText()
 
     # Reset the console namespace
@@ -337,7 +370,7 @@ def test_pylab_client(ipyconsole, qtbot):
         shell.execute("e")
 
     # Assert there are no errors after restting the console
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'Error' not in control.toPlainText()
 
 
@@ -357,7 +390,7 @@ def test_sympy_client(ipyconsole, qtbot):
         shell.execute("x")
 
     # Assert there are no errors in the console
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'NameError' not in control.toPlainText()
 
     # Reset the console namespace
@@ -368,8 +401,8 @@ def test_sympy_client(ipyconsole, qtbot):
     with qtbot.waitSignal(shell.executed):
         shell.execute("x")
 
-    # Assert there are no errors after restting the console
-    control = ipyconsole.get_focus_widget()
+    # Assert there are no errors after resetting the console
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'NameError' not in control.toPlainText()
 
 
@@ -393,7 +426,7 @@ def test_cython_client(ipyconsole, qtbot):
                       "    return x + y")
 
     # Assert there are no errors in the console
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'Error' not in control.toPlainText()
 
     # Reset the console namespace
@@ -407,7 +440,7 @@ def test_cython_client(ipyconsole, qtbot):
                       "    return x + y")
 
     # Assert there are no errors after restting the console
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     assert 'Error' not in control.toPlainText()
 
 
@@ -420,11 +453,11 @@ def test_tab_rename_for_slaves(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     cf = ipyconsole.get_current_client().connection_file
-    ipyconsole._create_client_for_kernel(cf, None, None, None)
+    ipyconsole.get_widget()._create_client_for_kernel(cf, None, None, None)
     qtbot.waitUntil(lambda: len(ipyconsole.get_clients()) == 2)
 
     # Rename slave
-    ipyconsole.rename_tabs_after_change('foo')
+    ipyconsole.get_widget().rename_tabs_after_change('foo')
 
     # Assert both clients have the same name
     assert 'foo' in ipyconsole.get_clients()[0].get_name()
@@ -438,11 +471,11 @@ def test_no_repeated_tabs_name(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Rename first client
-    ipyconsole.rename_tabs_after_change('foo')
+    ipyconsole.get_widget().rename_tabs_after_change('foo')
 
     # Create a new client and try to rename it
     ipyconsole.create_new_client()
-    ipyconsole.rename_tabs_after_change('foo')
+    ipyconsole.get_widget().rename_tabs_after_change('foo')
 
     # Assert the rename didn't take place
     client_name = ipyconsole.get_current_client().get_name()
@@ -459,7 +492,7 @@ def test_tabs_preserve_name_after_move(ipyconsole, qtbot):
     ipyconsole.create_new_client()
 
     # Move tabs
-    ipyconsole.tabwidget.tabBar().moveTab(0, 1)
+    ipyconsole.get_widget().tabwidget.tabBar().moveTab(0, 1)
 
     # Assert the second client is in the first position
     client_name = ipyconsole.get_clients()[0].get_name()
@@ -567,7 +600,7 @@ def test_console_disambiguation(ipyconsole, qtbot):
     ipyconsole.create_client_for_file(filename_d)
     client = ipyconsole.get_current_client()
     assert client.get_name() == 'c.py - d/A'
-    ipyconsole.tabwidget.setCurrentIndex(1)
+    ipyconsole.get_widget().tabwidget.setCurrentIndex(1)
     client = ipyconsole.get_current_client()
     assert client.get_name() == 'c.py - b/A'
 
@@ -578,14 +611,15 @@ def test_console_coloring(ipyconsole, qtbot):
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
-    config_options = ipyconsole.config_options()
+    config_options = ipyconsole.get_widget().config_options()
 
     syntax_style = config_options.JupyterWidget.syntax_style
     style_sheet = config_options.JupyterWidget.style_sheet
     console_font_color = get_console_font_color(syntax_style)
     console_background_color = get_console_background_color(style_sheet)
 
-    selected_color_scheme = CONF.get('appearance', 'selected')
+    selected_color_scheme = ipyconsole.get_conf(
+        'selected', section='appearance')
     color_scheme = get_color_scheme(selected_color_scheme)
     editor_background_color = color_scheme['background']
     editor_font_color = color_scheme['normal'][0]
@@ -713,7 +747,7 @@ def test_save_history_dbg(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Enter debugging mode
@@ -755,7 +789,7 @@ def test_save_history_dbg(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Enter debugging mode
@@ -792,7 +826,7 @@ def test_dbg_input(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Debug with input
@@ -848,10 +882,10 @@ def test_read_stderr(ipyconsole, qtbot):
 
     # Set contents of the stderr file of the kernel
     content = 'Test text'
-    stderr_file = client.stderr_file
+    stderr_file = client.stderr_obj.filename
     codecs.open(stderr_file, 'w', 'cp437').write(content)
     # Assert that content is correct
-    assert content == client._read_stderr()
+    assert content == client.stderr_obj.get_contents()
 
 
 @flaky(max_runs=10)
@@ -868,7 +902,7 @@ def test_values_dbg(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Enter debugging mode
@@ -918,7 +952,7 @@ def test_execute_events_dbg(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Import Matplotlib
@@ -930,7 +964,7 @@ def test_execute_events_dbg(ipyconsole, qtbot):
         shell.execute('%debug print()')
 
     # Set processing events to True
-    CONF.set('ipython_console', 'pdb_execute_events', True)
+    ipyconsole.set_conf('pdb_execute_events', True)
     shell.set_pdb_execute_events(True)
 
     # Test reset magic
@@ -942,7 +976,7 @@ def test_execute_events_dbg(ipyconsole, qtbot):
     assert shell._control.toHtml().count('img src') == 1
 
     # Set processing events to False
-    CONF.set('ipython_console', 'pdb_execute_events', False)
+    ipyconsole.set_conf('pdb_execute_events', False)
     shell.set_pdb_execute_events(False)
 
     # Test reset magic
@@ -1038,7 +1072,7 @@ def test_ctrl_c_dbg(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Enter debugging mode
@@ -1064,7 +1098,7 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Enter debugging mode
@@ -1124,7 +1158,7 @@ def test_load_kernel_file_from_id(ipyconsole, qtbot):
     connection_file = osp.basename(client.connection_file)
     id_ = connection_file.split('kernel-')[-1].split('.json')[0]
 
-    ipyconsole._create_client_for_kernel(id_, None, None, None)
+    ipyconsole.get_widget()._create_client_for_kernel(id_, None, None, None)
     qtbot.waitUntil(lambda: len(ipyconsole.get_clients()) == 2)
 
     new_client = ipyconsole.get_clients()[1]
@@ -1145,7 +1179,7 @@ def test_load_kernel_file_from_location(ipyconsole, qtbot, tmpdir):
     connection_file = to_text_string(tmpdir.join(fname))
     shutil.copy2(client.connection_file, connection_file)
 
-    ipyconsole._create_client_for_kernel(connection_file, None, None, None)
+    ipyconsole.get_widget()._create_client_for_kernel(connection_file, None, None, None)
     qtbot.waitUntil(lambda: len(ipyconsole.get_clients()) == 2)
 
     assert len(ipyconsole.get_clients()) == 2
@@ -1162,8 +1196,8 @@ def test_load_kernel_file(ipyconsole, qtbot, tmpdir):
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
 
-    ipyconsole._create_client_for_kernel(client.connection_file,
-                                         None, None, None)
+    ipyconsole.get_widget()._create_client_for_kernel(
+        client.connection_file, None, None, None)
     qtbot.waitUntil(lambda: len(ipyconsole.get_clients()) == 2)
 
     new_client = ipyconsole.get_clients()[1]
@@ -1192,7 +1226,6 @@ def test_sys_argv_clear(ipyconsole, qtbot):
 
 
 @flaky(max_runs=5)
-@pytest.mark.skipif(os.name == 'nt', reason="Fails on Windows")
 def test_set_elapsed_time(ipyconsole, qtbot):
     """Test that the IPython console elapsed timer is set correctly."""
     shell = ipyconsole.get_current_shellwidget()
@@ -1200,10 +1233,13 @@ def test_set_elapsed_time(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
 
+    # Show time label.
+    ipyconsole.get_widget().set_show_elapsed_time_current_client(True)
+
     # Set time to 2 minutes ago.
     client.t0 -= 120
     with qtbot.waitSignal(client.timer.timeout, timeout=5000):
-        ipyconsole.set_elapsed_time(client)
+        ipyconsole.get_widget().set_client_elapsed_time(client)
     assert ('00:02:00' in client.time_label.text() or
             '00:02:01' in client.time_label.text())
 
@@ -1235,9 +1271,9 @@ def test_stderr_file_is_removed_one_kernel(ipyconsole, qtbot, monkeypatch):
     # In a normal situation file should exist
     monkeypatch.setattr(QMessageBox, 'question',
                         classmethod(lambda *args: QMessageBox.Yes))
-    assert osp.exists(client.stderr_file)
+    assert osp.exists(client.stderr_obj.filename)
     ipyconsole.close_client(client=client)
-    assert not osp.exists(client.stderr_file)
+    assert not osp.exists(client.stderr_obj.filename)
 
 
 @flaky(max_runs=3)
@@ -1252,19 +1288,18 @@ def test_stderr_file_is_removed_two_kernels(ipyconsole, qtbot, monkeypatch):
                     timeout=SHELL_TIMEOUT)
 
     # New client with the same kernel
-    ipyconsole._create_client_for_kernel(client.connection_file, None, None,
-                                         None)
-
-    assert len(ipyconsole.get_related_clients(client)) == 1
-    other_client = ipyconsole.get_related_clients(client)[0]
-    assert client.stderr_file == other_client.stderr_file
+    ipyconsole.get_widget()._create_client_for_kernel(
+        client.connection_file, None, None, None)
+    assert len(ipyconsole.get_widget().get_related_clients(client)) == 1
+    other_client = ipyconsole.get_widget().get_related_clients(client)[0]
+    assert client.stderr_obj.filename == other_client.stderr_obj.filename
 
     # In a normal situation file should exist
     monkeypatch.setattr(QMessageBox, 'question',
                         classmethod(lambda *args: QMessageBox.Yes))
-    assert osp.exists(client.stderr_file)
+    assert osp.exists(client.stderr_obj.filename)
     ipyconsole.close_client(client=client)
-    assert not osp.exists(client.stderr_file)
+    assert not osp.exists(client.stderr_obj.filename)
 
 
 @flaky(max_runs=3)
@@ -1279,19 +1314,19 @@ def test_stderr_file_remains_two_kernels(ipyconsole, qtbot, monkeypatch):
                     timeout=SHELL_TIMEOUT)
 
     # New client with the same kernel
-    ipyconsole._create_client_for_kernel(client.connection_file, None, None,
-                                         None)
+    ipyconsole.get_widget()._create_client_for_kernel(
+        client.connection_file, None, None, None)
 
-    assert len(ipyconsole.get_related_clients(client)) == 1
-    other_client = ipyconsole.get_related_clients(client)[0]
-    assert client.stderr_file == other_client.stderr_file
+    assert len(ipyconsole.get_widget().get_related_clients(client)) == 1
+    other_client = ipyconsole.get_widget().get_related_clients(client)[0]
+    assert client.stderr_obj.filename == other_client.stderr_obj.filename
 
     # In a normal situation file should exist
     monkeypatch.setattr(QMessageBox, "question",
                         classmethod(lambda *args: QMessageBox.No))
-    assert osp.exists(client.stderr_file)
+    assert osp.exists(client.stderr_obj.filename)
     ipyconsole.close_client(client=client)
-    assert osp.exists(client.stderr_file)
+    assert osp.exists(client.stderr_obj.filename)
 
 
 @flaky(max_runs=3)
@@ -1330,20 +1365,21 @@ def test_kernel_crash(ipyconsole, qtbot):
 
 @flaky(max_runs=3)
 @pytest.mark.skipif(not os.name == 'nt', reason="Only works on Windows")
-def test_remove_old_stderr_files(ipyconsole, qtbot):
-    """Test that we are removing old stderr files."""
+def test_remove_old_std_files(ipyconsole, qtbot):
+    """Test that we are removing old std files."""
     # Create empty stderr file in our temp dir to see
     # if it's removed correctly.
     tmpdir = get_temp_dir()
     open(osp.join(tmpdir, 'foo.stderr'), 'a').close()
 
     # Assert that only that file is removed
-    ipyconsole._remove_old_stderr_files()
+    ipyconsole._remove_old_std_files()
     assert not osp.isfile(osp.join(tmpdir, 'foo.stderr'))
 
 
 @flaky(max_runs=10)
 @pytest.mark.use_startup_wdir
+@pytest.mark.skipif(os.name == 'nt', reason="Too flaky on Windows")
 def test_console_working_directory(ipyconsole, qtbot):
     """Test for checking the working directory."""
     shell = ipyconsole.get_current_shellwidget()
@@ -1367,7 +1403,7 @@ def test_console_complete(ipyconsole, qtbot, tmpdir):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     def check_value(name, value):
@@ -1499,7 +1535,7 @@ def test_pdb_multiline(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     with qtbot.waitSignal(shell.executed):
@@ -1531,11 +1567,11 @@ def test_pdb_ignore_lib(ipyconsole, qtbot, show_lib):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Tests assume inline backend
-    CONF.set('ipython_console', 'pdb_ignore_lib', not show_lib)
+    ipyconsole.set_conf('pdb_ignore_lib', not show_lib)
     with qtbot.waitSignal(shell.executed):
         shell.execute('%debug print()')
 
@@ -1552,7 +1588,7 @@ def test_pdb_ignore_lib(ipyconsole, qtbot, show_lib):
         assert 'iostream.py' in control.toPlainText()
     else:
         assert 'iostream.py' not in control.toPlainText()
-    CONF.set('ipython_console', 'pdb_ignore_lib', True)
+    ipyconsole.set_conf('pdb_ignore_lib', True)
 
 
 @flaky(max_runs=3)
@@ -1568,7 +1604,7 @@ def test_calltip(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = {"a": 1}')
@@ -1644,7 +1680,7 @@ def test_wrong_std_module(ipyconsole, qtbot, tmpdir, spyder_pythonpath):
         wrong_random_mod = tmpdir.join('random.py')
         wrong_random_mod.write('')
         wrong_random_mod = str(wrong_random_mod)
-        CONF.set('main', 'spyder_pythonpath', [str(tmpdir)])
+        ipyconsole.set_conf('spyder_pythonpath', [str(tmpdir)], section='main')
     else:
         wrong_random_mod = osp.join(os.getcwd(), 'random.py')
         with open(wrong_random_mod, 'w') as f:
@@ -1672,7 +1708,7 @@ def test_wrong_std_module(ipyconsole, qtbot, tmpdir, spyder_pythonpath):
     os.remove(wrong_random_mod)
 
     # Restore CONF
-    CONF.set('main', 'spyder_pythonpath', [])
+    ipyconsole.set_conf('spyder_pythonpath', [], section='main')
 
 
 @flaky(max_runs=3)
@@ -1722,11 +1758,40 @@ def test_stderr_poll(ipyconsole, qtbot):
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
     client = ipyconsole.get_current_client()
-    with open(client.stderr_file, 'w') as f:
+    client.stderr_obj.handle.flush()
+    with open(client.stderr_obj.filename, 'a') as f:
         f.write("test_test")
     # Wait for the poll
-    qtbot.wait(2000)
-    assert "test_test" in ipyconsole.get_focus_widget().toPlainText()
+    qtbot.waitUntil(lambda: "test_test" in ipyconsole.get_widget(
+        ).get_focus_widget().toPlainText())
+    assert "test_test" in ipyconsole.get_widget(
+        ).get_focus_widget().toPlainText()
+    # Write a second time, makes sure it is not duplicated
+    client.stderr_obj.handle.flush()
+    with open(client.stderr_obj.filename, 'a') as f:
+        f.write("\ntest_test")
+    # Wait for the poll
+    qtbot.waitUntil(lambda: ipyconsole.get_widget().get_focus_widget(
+        ).toPlainText().count("test_test") == 2)
+    assert ipyconsole.get_widget().get_focus_widget().toPlainText(
+        ).count("test_test") == 2
+
+
+@flaky(max_runs=3)
+def test_stdout_poll(ipyconsole, qtbot):
+    """Test if the content of stdout is printed to the console."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+    client = ipyconsole.get_current_client()
+    client.stdout_obj.handle.flush()
+    with open(client.stdout_obj.filename, 'a') as f:
+        f.write("test_test")
+    # Wait for the poll
+    qtbot.waitUntil(lambda: "test_test" in ipyconsole.get_widget(
+        ).get_focus_widget().toPlainText(), timeout=5000)
+    assert "test_test" in ipyconsole.get_widget().get_focus_widget(
+        ).toPlainText()
 
 
 @flaky(max_runs=10)
@@ -1738,12 +1803,14 @@ def test_startup_code_pdb(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     # Run a line on startup
-    CONF.set('ipython_console', 'startup/pdb_run_lines',
-             'abba = 12; print("Hello")')
+    ipyconsole.set_conf(
+        'startup/pdb_run_lines',
+        'abba = 12; print("Hello")'
+    )
 
     shell.execute('%debug print()')
     qtbot.waitUntil(lambda: 'Hello' in control.toPlainText())
@@ -1752,7 +1819,7 @@ def test_startup_code_pdb(ipyconsole, qtbot):
     assert shell.get_value('abba') == 12
 
     # Reset setting
-    CONF.set('ipython_console', 'startup/pdb_run_lines', '')
+    ipyconsole.set_conf('startup/pdb_run_lines', '')
 
 
 @flaky(max_runs=3)
@@ -1771,7 +1838,7 @@ def test_pdb_eventloop(ipyconsole, qtbot, backend):
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
 
     with qtbot.waitSignal(shell.executed):
         shell.execute("%matplotlib " + backend)
@@ -1789,7 +1856,7 @@ def test_recursive_pdb(ipyconsole, qtbot):
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
 
     with qtbot.waitSignal(shell.executed):
         shell.execute("%debug print()")
@@ -1838,8 +1905,8 @@ def test_stop_pdb(ipyconsole, qtbot):
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
-    control = ipyconsole.get_focus_widget()
-    stop_button = ipyconsole.get_current_client().stop_button
+    control = ipyconsole.get_widget().get_focus_widget()
+    stop_button = ipyconsole.get_widget().stop_button
     # Enter pdb
     with qtbot.waitSignal(shell.executed):
         shell.execute("%debug print()")
@@ -1871,7 +1938,7 @@ def test_code_cache(ipyconsole, qtbot):
                     timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
     control.setFocus()
 
     def check_value(name, value):
@@ -1923,7 +1990,7 @@ def test_pdb_code_and_cmd_separation(ipyconsole, qtbot):
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
-    control = ipyconsole.get_focus_widget()
+    control = ipyconsole.get_widget().get_focus_widget()
 
     with qtbot.waitSignal(shell.executed):
         shell.execute("%debug print()")
@@ -1946,5 +2013,33 @@ def test_pdb_code_and_cmd_separation(ipyconsole, qtbot):
     assert "Unknown command 'abba'" in control.toPlainText()
 
 
+@flaky(max_runs=3)
+def test_breakpoint_builtin(ipyconsole, qtbot, tmpdir):
+    """Check that the breakpoint builtin is working."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+    control = ipyconsole.get_widget().get_focus_widget()
+
+    # Code to run
+    code = dedent("""
+    print('foo')
+    breakpoint()
+    """)
+
+    # Write code to file on disk
+    file = tmpdir.join('test_breakpoint.py')
+    file.write(code)
+
+    # Run file
+    with qtbot.waitSignal(shell.executed):
+        shell.execute(f"runfile(filename=r'{str(file)}')")
+
+    qtbot.wait(5000)
+    # Assert we entered debugging after the print statement
+    assert 'foo' in control.toPlainText()
+    assert 'IPdb [1]:' in control.toPlainText()
+
+
 if __name__ == "__main__":
     pytest.main()
